/*

  File LUmat.cpp:

  contiene la definizione delle funzioni membro della classe LUmat, che
  definisce una matrice generica nxn, la fattorizza LU e risolve m sistemi
  di dimensione n con la stessa matrice ma differenti secondi membri.

*/


#include "LUmat.h"


/*

 LUmat(): costruttore della classe

*/

LUmat::LUmat( int N )
{
  dim = N;
  mat = new double * [N];
  for ( int i=0; i < N; i++ ) mat[i] = new double[N];
  tmp = new double [N];
}


/*

  ~LUmat(): distruttore della classe LUmat.

*/

LUmat::~LUmat()
{
  for ( int i = 0; i < dim; i++ ) delete( mat[i] );
  delete( mat );
  delete( tmp );
}



/*

  luFact( ): effettua la fattorizzazione LU di una matrice a[dim][dim]

*/


void LUmat::luFact( )
{

  double sum;

  for ( int j = 1; j < dim ; j++ ) mat[0][j] = mat[0][j] / mat[0][0];

  for ( int i = 1; i < dim; i++ )
    {

      for ( int j = 1; j <= i; j++ )
	{
	  sum = 0.0;
	  for ( int k = 0; k < j; k++ ) sum += mat[i][k] * mat[k][j];
	  mat[i][j] = mat[i][j] - sum;
	}

      for ( int j = i + 1; j < dim; j++ )
	{
	  sum = 0.0;
	  for ( int k = 0; k < i; k++ ) sum += mat[i][k] * mat[k][j];
	  mat[i][j] = ( mat[i][j] - sum ) / mat[i][i];
	}

    }

}


/*

  luSolve( x, b ): esegue l'inversione di un sistema a*x=b

*/


void LUmat::luSolve( double * x, double * b )
{

  double sum;

  // Forward substitution
  x[0] = b[0] / mat[0][0];    // Primo elemento
  for ( int i = 1; i < dim; i++ )
    {
      sum = 0.0;
      for ( int j = 0; j < i; j++ ) sum += mat[i][j] * x[j];
      x[i] = ( b[i] - sum ) / mat[i][i];
    }

  // Ricopio il vettore soluzione della FS in tmp

  for ( int i = 0; i < dim; i++ ) tmp[i] = x[i];

  // Backward substitution
  x[dim-1] = tmp[dim-1];
  for ( int i = dim - 2; i >= 0; i-- )
    {
      sum = 0.0;
      for ( int j = i + 1; j < dim; j++ ) sum += mat[i][j] * x[j];
      x[i] = tmp[i] - sum;
    }

}


/*

   Funzione setMatrix:
   definisce la matrice a partire da un'altra matrice passata tra gli argomenti

*/

void LUmat::setMatrix( double * a )
{
  double * p = a;
  for ( int i = 0; i < dim; i++ )
    for ( int j = 0; j < dim; j++ )
      mat[i][j] = *p++;
}


/*

  Funzione setElement:
  inizializza l'elemento a_ij della matrice al valore passato

*/

void LUmat::setElement( int i, int j, double a_ij )
{
  if ( ( ( i >= 0 ) && ( i < dim ) ) && ( ( j >= 0 ) && ( j < dim ) ) )
    mat[i][j] = a_ij;
}


/*

  Funzione getElement:
  restituisce il valore dell'elemento i,j della matrice

*/

double LUmat::getElement( int i, int j )
{
  if ( ( ( i >= 0 ) && ( i < dim ) ) && ( ( j >= 0 ) && ( j < dim ) ) )
    return( mat[i][j] );
}
