/*

  File Sis_ndd.ccp:

  Esattamente identico a sis_scal.cpp, solo, per mostrare l'effetto della imprecisione negli
  arrotondamenti nei calcoli, nel caso in cui la matrice NON sia diagonalmente dominante,
  rimuovo la condizione: a_ij = 1 se i = j.
  La matrice viene costruita come:
  a_ij = 1.0/(1.0+i+j)

  Provare, ad esempio, il caso: N = 3, N = 8 e N = 20 per vedere come 
  l'approssimazione peggiora all'aumentare di N. Nota che per N=50 il programma va in overflow,
  arrivandoci abbastanza improvvisamente (N=47 da' ancora un risultato abbastanza preciso!)

*/


#include <iostream>
#include "LUmat.h"
using namespace std;


main()
{
  cout << "Calcolo della soluzione di un sistema nxn tramite fattorizzazione LU" << endl;

  // Definizione della matrice a.

  const int N = 20;
  double mat[N][N];
  for ( int i = 0; i < N; i++ )
    for ( int j = 0; j < N; j++ )
      {
	mat[i][j]  = 1.0 / ( 1.0 + (i+1) + (j+1) );
      }


  // Stampa sullo schermo la matrice iniziale

  cout.precision( 15 );     // Setta la precisione di stampa con 16 cifre decimali!
  cout << "Matrice iniziale:" << endl;
  for ( int i = 0; i < N; i++ )
    {
      for ( int j = 0; j < N; j++ ) cout << "   " << mat[i][j];
      cout << endl;
    }

  // Crea l'oggetto matrice

  LUmat matrice = LUmat( N );

  // Lo inizializza ed esegue la fattorizzazione LU della matrice

  matrice.setMatrix( &mat[0][0] );
  matrice.luFact( );


  // Costruisce i secondi membri del sistema

  double x[N];
  double sm[N];

  for ( int i = 0; i < N; i++ )
    {
      sm[i] = i * 1.0;
    }


  // Stampa sullo schermo il vettore del secondo membro

  cout << "Vettore del secondo membro:" << endl;
  for ( int i = 0; i < N; i++ ) cout << "   " << sm[i];
  cout << endl;

  // Inverte il sistema fattorizzato LU

  matrice.luSolve( x, sm );


  // Stampa del vettore soluzione

  cout << "Stampa del risultato." << endl;
  cout << "Vettore soluzione:" << endl;
  for ( int i = 0; i < N; i++ ) cout << x[i] << "   ";
  cout << endl;

  // Verifica del risultato

  cout << "Verifica del risultato." << endl;
  cout << "Prodotto matrice per vettore soluzione:" << endl;
  for ( int i = 0; i < N; i++ )
    {
      double sum = 0.0;
      for ( int j = 0; j < N; j++ ) sum += mat[i][j] * x[j];
      cout << "   " << sum;
    }
  cout << endl;

}
